import React from "react";
import { Link } from "react-router-dom";
import { fetchJSON } from "../../../../utils.js";

import DocumentPageFrame from "../../../UI/DocumentPageFrame";
import Crumbs from "../../../UI/Crumbs";

import ConditionalItemField from "../../../UI/ConditionalItemField"
import MediaElement from "./MediaElement"

import Button from '@material-ui/core/Button';

import FormControl from '@material-ui/core/FormControl';
import Select from '@material-ui/core/Select';
import InputLabel from '@material-ui/core/InputLabel';
import MenuItem from '@material-ui/core/MenuItem';

const constants = require("../../../../constants.js");

export default class MediaItem extends React.Component {

  constructor(props) {
    super(props);
    this.state = {
      doc: null,
      playbackRates: []
    };

  }


  componentDidUpdate(prevProps){
    const item_id = this.props.match.params.id;
    if (prevProps.match.params.id !== item_id){
      this.fetchData(item_id);
    }
  }


  componentDidMount() {
    this.fetchData(this.props.match.params.id);
  }


  componentWillUnmount(){
    if (document) {
      document.title = constants.APP_TITLE;
    }
  }


  fetchData(item_id) {

    const self = this;

    if (!item_id){
      self.setState({
        error: "unknownID"
      });
      return;
    }
    const url = constants.URLS.DATABASE_API + "AV/item/" + item_id;

    self.setState({
      isLoading: true,
      doc: null
    });

    fetchJSON(url)
    .then(function(json){

      if (json.error){
        self.setState({
          error: "unknownID",
          isLoading: false
        });
        return;
      }

      self.setState({
        doc: json,
        isLoading: false
      });

      if (document) {
        document.title = json.title
        + " - "
        + constants.APP_TITLE;
      }

    })
    .catch(e => {
      self.setState({
        error: "server",
        isLoading: false
      });
      console.log(e);
    });

    return;

  }


  renderMediaElements(doc){
    if (doc.mediaType === "Video"){
      return doc.parts.map((part, i) => {
        return <MediaElement
          type="video"
          files={part.files}
          partIndex={i}
          key={"me-" + i}
        />;
      });
    } else {
      return doc.parts.map((part, i) => {
        return <MediaElement
          type="audio"
          files={part.files}
          partIndex={i}
          key={"me-" + i}
        />;
      });
    }
  }


  jumpToTimecode(partIndex, timecode){
    const mediaElement = document.getElementById("mediaElement-" + partIndex);

    if(!mediaElement){
      console.log("ERROR: mediaElement not found: " + partIndex);
    }

    const minutes = parseInt(timecode.split(":")[0]);
    const seconds = parseInt(timecode.split(":")[1]);
    const time = minutes * 60 + seconds;

    mediaElement.currentTime = time;
    mediaElement.play();
  }


  renderJumpLabels(doc){
    const markup = [];
    for (let p = 0; p < doc.parts.length; p++){
      const part = doc.parts[p];
      for (let j = 0; j < part.jumpLabels.length; j++){
        const jumpLabel = part.jumpLabels[j];
        markup.push(
          <p key={"jl-" + p + "-" + j}>
            <Button
              variant="contained"
              color="primary"
              onClick={() => this.jumpToTimecode(p, jumpLabel.timecode)}
            >
              {jumpLabel.timecode + " - " + jumpLabel.label}
            </Button>
          </p>
        );
      }
    }

    if (markup.length > 0){
      return <>
        <h2>Sprungmarken</h2>
        {markup}
      </>
    } else {
      return "";
    }
  }


  renderRelatedDoc(rd, i){
    const LABELS = {
      "BIBL": "Bibliographischer Eintrag",
      "ZK": "Zettel",
      "MS": "Manuskript"
    }

    const LINK_PREFIXES = {
      "AV": "/bestand/av/item/",
      "ZK": "/bestand/zettelkasten/zettel/",
      "BIBL": "/bestand/bibliographie/item/"
    }

    let headline;
    if (rd.id.length > 0 && rd.type !== "MS"){
      headline = <Link to={LINK_PREFIXES[rd.type] + rd.id}>
        {LABELS[rd.type] + " " + rd.id}
      </Link>
    } else {
      headline = <strong>{LABELS[rd.type]}</strong>;
    }

    return <div key={"rd_" + i}>
    <p>
      {headline}
      <br/>
      {rd.description}
    </p>
  </div>
  }


  render(){

    if (!this.state.doc) return "";

    const doc = this.state.doc;

    return (
      <DocumentPageFrame>
        <Crumbs crumbs={[
          {
            "link": "/",
            "title": "Niklas-Luhmann-Archiv"
          },
          {
            "link": "/bestand",
            "title": "Bestand"
          },
          {
            "link": "/bestand/av/suche",
            "title": "Audio / Video"
          },
          {
            "link": "/bestand/av/video/" + doc.id,
            "title": doc.title
          }
        ]} />
        <h1>{doc.title}</h1>
        {this.renderMediaElements(doc)}
        <h2>Beschreibung</h2>
        {doc.description.split("\n").map((p, i) => <p key={"dp_" + i}>{p}</p>)}
        {this.renderJumpLabels(doc)}
        <h2>Fundstellen im Portalbestand</h2>
        {
          doc.relatedDocuments.map(this.renderRelatedDoc)
        }
        <h2>Metadaten</h2>
        <ConditionalItemField label="Titel" value={doc.title} />
        <ConditionalItemField label="Kurztitel" value={doc.shortTitle} />
        <ConditionalItemField label="Untertitel" value={doc.subtitle} />
        <ConditionalItemField label="Typ" value={doc.type} />
        <ConditionalItemField label="Medientyp" value={doc.mediaType} />
        <ConditionalItemField label="Quelle" value={doc.source} />
        <ConditionalItemField
          label="Rechteinhaber"
          value={doc.legal.rightsHolder}
        />
        <ConditionalItemField
          label="Datum (Aufzeichnung)"
          value={doc.dateOfRecording}
        />
        <ConditionalItemField
          label="Datum (Sendung/VÖ)"
          value={doc.dateOfPublication}
        />
        <ConditionalItemField label="ID" value={doc.id} />

        <h2>Downloads</h2>
        <a
          href={constants.URLS.DATABASE_API + "AV/item/" + doc.id + "/xml"}
          className="buttonLink"
          style={{"margin": "5px"}}
        >
          <Button color="primary">
            XML
          </Button>
        </a>
        <a
          href={constants.URLS.DATABASE_API + "AV/item/" + doc.id}
          className="buttonLink"
          style={{"margin": "5px"}}
        >
          <Button color="primary">
            JSON
          </Button>
        </a>
      </DocumentPageFrame>
    );
  }
}
