import React from "react";

import { Link } from "react-router-dom";
import { fetchJSON, groupByKey } from "../../../../../../utils.js";

import DocumentPageFrame from "../../../../../UI/DocumentPageFrame";
import BusySpinner from "../../../../../UI/BusySpinner";
import Crumbs from "../../../../../UI/Crumbs";

import ConditionalItemField from "../../../../../UI/ConditionalItemField"
import DetailInfo from "./DetailInfo/"
import Metadata from "./Metadata"
import Error from "./Error.js";

import Button from '@material-ui/core/Button';

const constants = require("../../../../../../constants.js");
import * as BiblLogic from "../../../../../../BiblLogic.js";

import withWidth from '@material-ui/core/withWidth';

class Item extends React.Component {

  constructor(props) {
    super(props);
    this.state = {
      item: null,
      isLoading: false,
      error: null
    };
  }

  renderPtrs(ptrs, isGesamtbibliographie){

    if (isGesamtbibliographie){
      ptrs = [
        {
          n: "Niklas-Luhmann-Gesamtbibliographie",
          type: "nl_gesamtbiblio"
        },
        ...ptrs
      ];
    }


    if ((!ptrs) || ptrs.length === 0){
      return <p>Keine Projektquellen vorhanden</p>;
    }

    const listItems = ptrs.map((ptr, i) => {

      if (ptr.type === "ZK"){
        return (
          <li key={"ptr_" + i}>
            <Link
              to={"/bestand/zettelkasten/zettel/" + ptr.target}
              title={ptr.target}
            >
              {ptr.n}
            </Link>
          </li>
        );
      } else if (ptr.type === "AV"){
        return (
          <li key={"ptr_" + i}>
            <Link
              to={"/bestand/av/item/" + ptr.target}
              title={ptr.target}
            >
              {(ptr.mediaType === "video" ? "Video" : "Audio") + ": " + ptr.n}
            </Link>
          </li>
        );
      } else if (ptr.type === "nl_gesamtbiblio"){
        return (
          <li key={"ptr_" + i}>
            <Link to={"/bestand/nl-gesamtbibliographie"}>
              {ptr.n}
            </Link>
          </li>
        );
      } else {
        return "";
      }

    });

    const width = this.props.width;
    const isSmallScreen = width === "xs" || width === "sm";
    const isAtLeastMediumScreen =
      width === "md" || width === "lg" || width === "xl";

    let columns = 1;
    if (listItems.length > 5 && isAtLeastMediumScreen) columns = 2;
    if (listItems.length > 12 && isAtLeastMediumScreen) columns = 3;

    return <ul style={{"columns": columns.toString()}}>
      {listItems}
    </ul>;

  }


  componentWillUnmount(){
    if (document) {
      document.title = constants.APP_TITLE;
    }
  }


  renderNotes(notes){
    return notes.map((note, i) => {
      return (
        <span key={"note"+i}>
          <span
            dangerouslySetInnerHTML={{"__html": note.html}}
          />
          <br/>
        </span>
      );
    });
  }


  renderRelatedItems(relatedItems){
    if ((!relatedItems) || relatedItems.length === 0){
      return "Keine Einträge vorhanden";
    }

    //this sorts the relatedItems after the order of BiblLogic.relations
    const relatedItemsSorted = relatedItems
    .sort((a, b) => {
      const relationA = a.type;
      const relationB = b.type;

      const indexOfRelationA = BiblLogic.relations
      .findIndex(r => r.id === relationA);

      const indexOfRelationB = BiblLogic.relations
      .findIndex(r => r.id === relationB);

      return indexOfRelationA - indexOfRelationB;
    });

    const relatedItemsGrouped = groupByKey(relatedItemsSorted, "type");

    const markup = [];
    let i = 0;

    for (let type in relatedItemsGrouped) {
      relatedItemsGrouped[type].forEach((ri) => {
        markup.push(
          <li key={"ri-" + i}>
            <span dangerouslySetInnerHTML={{"__html": ri.html}}>
            </span>
          </li>
        );
        i++;
      });
      //insert gap between different relation types
      markup.push(<p key={"ri-gap-" + i}> </p>);
    }

    return <ul>{markup}</ul>;
  }


  fetchData(id){

    const self = this;

    if (!id){
      self.setState({
        error: "unknownID"
      });
      return;
    }

    let url = constants.URLS.DATABASE_API + "BIBL/item/" + id;

    self.setState({
      isLoading: true,
      item: null,
      error: null
    });

    fetchJSON(url)
    .then(json => {

      if (json.error){
        self.setState({
          error: "unknownID",
          isLoading: false
        });
        return;
      }

      self.setState({
        item: json,
        error: null,
        isLoading: false
      });

      if (document && this.state.item.id) {
        document.title = this.state.item.id
        + " - "
        + constants.APP_TITLE;
      }

    })
    .catch(e => {
      self.setState({
        error: "server",
        isLoading: false
      });
      console.log(e);
    });

    return;

  }


  componentDidUpdate(prevProps){
    const item_id = this.props.match.params.id;
    if (prevProps.match.params.id !== item_id){
      this.fetchData(item_id);
    }
  }


  componentDidMount() {
    this.fetchData(this.props.match.params.id);
  }


  render(){

    if (this.state.error){
      return <Error
        error={this.state.error}
        id={this.props.match.params.id || ""}
      />;
    }

    const self = this;
    const item = this.state.item;

    if (!item){
      return (
        <DocumentPageFrame>
          <BusySpinner/>
        </DocumentPageFrame>
      );
    }

    return (
      <DocumentPageFrame>
        <Crumbs crumbs={[
          {
            "link": "/",
            "title": "Niklas-Luhmann-Archiv"
          },
          {
            "link": "/bestand",
            "title": "Bestand"
          },
          {
            "link": '/bestand/register/bibliographie',
            "title": "Bibliographie"
          },
          {
            "link": "/bestand/bibliographie/item/" + item.id,
            "title": item.id
          },
        ]} />
        <h1 style={{"wordBreak": "break-all"}}>{item.display.short}</h1>
        <p style={{
          "fontWeight": "900",
          "fontSize": "1.2rem"
        }}>{item.display.full}</p>

        <h2>Fundstellen im Portalbestand</h2>
        {this.renderPtrs(item.ptrs, item.isGesamtbibliographie)}

        <DetailInfo type={this.type} item={item}/>

        {item.notes.length > 0 ?
          <div>
            <h2>Anmerkungen</h2>
            {this.renderNotes(item.notes)}
          </div> : ""
        }

        <h2>Zugehörige bibliographische Einträge</h2>
        {this.renderRelatedItems(item.relatedItems)}

        <Metadata item={item}/>

        <h2>Downloads</h2>
        <a
          href={constants.URLS.DATABASE_API + "BIBL/item/" + item.id + "/xml"}
          className="buttonLink"
          style={{"margin": "5px"}}
        >
          <Button color="primary">
            TEI-XML
          </Button>
        </a>
        <a
          href={constants.URLS.DATABASE_API + "BIBL/item/" + item.id}
          className="buttonLink"
          style={{"margin": "5px"}}
        >
          <Button color="primary">
            JSON
          </Button>
        </a>

      </DocumentPageFrame>
    );
  }
}

export default withWidth()(Item);
