import React from "react";

import { Link } from "react-router-dom";
import { fetchJSON } from "../../../../../utils.js";
const uuidv4 = require('uuid/v4');

//Shared UI
import FullPageFrame from "../../../../UI/FullPageFrame";
import BusySpinner from "../../../../UI/BusySpinner";
import Crumbs from "../../../../UI/Crumbs";
import Pagination from "../../../../UI/Pagination";

//Material-UI
import { withStyles } from '@material-ui/core/styles';
import AppBar from '@material-ui/core/AppBar';
import Typography from '@material-ui/core/Typography';
import Grid from '@material-ui/core/Grid';
import Paper from '@material-ui/core/Paper';
import AccountCircle from '@material-ui/icons/AccountCircle';

import ZKFilters from "./Filters";
import ZKResults from "./Results";

const constants = require("../../../../../constants.js");

function TabContainer(props) {
  return (
    <Typography component="div" style={{ padding: 8 * 3 }}>
      {props.children}
    </Typography>
  );
}


export default class Search extends React.Component {

  constructor(props) {
    super(props);
    this.state = {
      results: null,
      numberOfResults: null,
      resultsType: null
    }
  }


  getURLState(url){

    if (typeof url !== "string"){
      url = this.props.location.search;
    }

        let queryString =
          url && url.substring(0, 3) === "?q="
          ? url.substring(3)
          : "{}";

        let query;
        try {
          query = JSON.parse(decodeURI(queryString));
        } catch (e) {
          //if the q param is no valid JSON, treat it as string for fulltext search
          query = {fulltext: queryString};
        }

        //Default value for page
        if (typeof query.page !== "number") query.page = 1;
        //Default value for fulltext
        if (typeof query.fulltext !== "string") query.fulltext = "";
        //Default value for fuzzy
        if (typeof query.fuzzy !== "boolean") query.fuzzy = false;
        //Default value for FTSearchMode
        if (typeof query.FTSearchMode !== "string") query.FTSearchMode = "and";
        //Default value for zettelnummer
        if (typeof query.zettelnummer !== "string") query.zettelnummer = "";
        //Default value for zettelnummerSearchMode
        if (typeof query.zettelnummerSearchMode !== "string"){
          query.zettelnummerSearchMode = "exact";
        }
        //Default value for areas
        if (!Array.isArray(query.areas)) query.areas = [];

        return query;
  }


  componentDidMount() {
    this.fetchData();
    document.title = "Zettelsuche" + " - " + constants.APP_TITLE;
  }


  componentWillUnmount(){
    if (document) {
      document.title = constants.APP_TITLE;
    }
  }


  componentDidUpdate(previousProps, previousState){
    const currentQuery = this.getURLState();
    const oldSearch = previousProps.location.search;
    const oldQuery = this.getURLState(oldSearch);

    const oldQueryString = JSON.stringify(oldQuery);
    const currentQueryString = JSON.stringify(currentQuery);

    if (oldQueryString !== currentQueryString){
      this.fetchData();
    }
  }


  setURLState(query){
    const location = {...this.props.history.location};
    location.search = "q=" + encodeURI(JSON.stringify(query));
    this.props.history.push(location);
  }


  // Take the query object from the current state and request
  // the data from the server
  fetchData(){
    const self = this;
    const query = this.getURLState();

    //add the pre-defined hard-coded rows value for the query
    query.rows = constants.SEARCH_RESULTS_PER_PAGE;

    const url = constants.URLS.DATABASE_API
    + `ZK/search`
    + "?q=" + encodeURI(JSON.stringify(query))
    + "&client_id=" + constants.CLIENT_ID;

    const fetchID = uuidv4();

    this.setState({
      isFetching: true,
      results: null,
      numberOfResults: null,
      resultsType: null,
      fetchID
    });

    fetchJSON(url).then(function(json) {
      if (self.state.fetchID === fetchID){
        self.setState({
          results: json.results,
          numberOfResults: json.numberOfResults,
          isFetching: false,
          fetchID: null
        });
      }
    })
    .catch(e => console.log(e));
  }


  setPage(activePageNumber){
    const oldQuery = this.getURLState();

    const newQuery = {
      ...oldQuery,
      page: activePageNumber
    };

    this.setURLState(newQuery);
  }


  handleFilterChange(key, newValue){
    const oldQuery = this.getURLState();

    const newQuery = {
      ...oldQuery
    };

    newQuery[key] = newValue;

    //Reset page on filter change
    newQuery.page = 1;

    this.setURLState(newQuery);
  }


  render(){
    const self = this;
    const state = this.state;
    const filterState = this.getURLState();

    const FilterComponent = ZKFilters;
    const ResultsComponent = ZKResults;

    return (
      <FullPageFrame>
        <div style={{
          "display": "grid",
          "gridTemplateRows": "auto",
          "gridTemplateColumns": "1fr 3fr",
          "gridGap": "0px 50px"
        }}>
          <div className="filters" style={{"gridArea": "1 / 1 / 2 / 2"}}>
            <FilterComponent
              onChange={(key, value) => this.handleFilterChange(
                key, value
              )}
              state={filterState}
            />
          </div>
          <div className="results" style={{"gridArea": "1 / 2 / 2 / 3"}}>
            <Paper>
              <div style={{"padding": "15px"}}>
                {((!state.results) || state.isFetching)
                  ? <BusySpinner/>
                  :
                  <div>
                    <Pagination
                      numberOfResults={state.numberOfResults}
                      activePage={filterState.page}
                      onChange={(i) => this.setPage(i)}
                      searchResultsPerPage={constants.SEARCH_RESULTS_PER_PAGE}
                    />
                    <div style={{"margin": "15px 0px"}}>
                      <ResultsComponent
                        results={state.results}
                        tokensToHighlight={
                          filterState.fulltext
                          && filterState.fulltext.split(" ")
                          || []
                        }
                      />
                    </div>
                    <Pagination
                      numberOfResults={state.numberOfResults}
                      activePage={filterState.page}
                      onChange={(i) => this.setPage(i)}
                      searchResultsPerPage={constants.SEARCH_RESULTS_PER_PAGE}
                    />
                  </div>
                }
              </div>
            </Paper>
          </div>
        </div>
      </FullPageFrame>
    );
  }
}
