import React from "react";

import { Link } from "react-router-dom";
import * as d3 from "d3-selection";
import { fetchJSON } from "../../../../../utils.js";

const constants = require("../../../../../constants.js");

export default class BranchViewBox extends React.Component {

  constructor(props) {
    super(props);
    this.state = {
      visAvailable: false,
      vis_data: null
    };
  }


  componentDidUpdate(prevProps){

    const props = this.props;

    if (!(
      props
      && props.zettel
      && props.zettel.zk_number
      && props.zettel.flags.hasBranchVisualization
    )) {
      return;
    }

    if (
      (!prevProps.zettel)
      || (prevProps.zettel.ekin !== props.zettel.ekin)
    ){
      this.fetchData(props.zettel);
    }
  }


  componentDidMount() {

    const props = this.props;

    if (!(
      props
      && props.zettel
      && props.zettel.zk_number
      && props.zettel.flags.hasBranchVisualization
    )) {
      return;
    }

    this.fetchData(props.zettel);
  }


  buildFetchURLFromZettelObject(zettel){
    const file = zettel.file;
    const dir_zettelkasten = file["dir-zettelkasten"];
    const dir_auszug = file["dir-auszug"];
    const dir_batch = file["dir-batch"];
    const image_id = zettel.file.id;

    const url = constants.URLS.BRANCH_VIS_DATA_FOLDER
    + dir_zettelkasten + "/"
    + dir_auszug + "/"
    + dir_batch + "/"
    + image_id
    + ".json";

    return url;

  }


  fetchData(zettel) {

    const self = this;

    const url = this.buildFetchURLFromZettelObject(zettel);

    fetchJSON(url).then(function(json){
      self.setState({
        vis_data: json,
        visAvailable: true
      });
      self.createVisualization(json);
    }).catch(function(e){
      console.log(e);
      self.setState({
        visAvailable: false
      });
    });

    return;

  }


  createVisualization(zettel){
    const ZETTEL_URL_PREFIX = constants.URLS.ZETTEL_URL_PREFIX;
    const MAXIMUM_HORIZONTAL_NODES = 15;
    const distanceX = 80; //left right
    const distanceY = 70; //up down

    var collectNodesAndStubs = function(node, referenceNode, nodes, stubs){

      if (!node) return;
      nodes.push(node);
      node.subNodes && node.subNodes.forEach(subNode => {
        collectNodesAndStubs(subNode, node, nodes, stubs);
      });
      node.nextNode && collectNodesAndStubs(node.nextNode, node, nodes, stubs);
      node.previousNode && collectNodesAndStubs(node.previousNode, node, nodes, stubs);
      node.originNode && collectNodesAndStubs(node.originNode, node, nodes, stubs);
      node.referenceNode = referenceNode;

      node.stubs && node.stubs.forEach(stub => {
        stubs.push({
          ...stub,
          origin: node
        });
      });

      return {
        nodes,
        stubs
      };

    };


    var assignWidthAndHeight = function(nodes, offsetX, offsetY){
      nodes.forEach(function(node){
        node.x = node.x * distanceX + offsetX;
        node.y = node.y * distanceY + offsetY;
      });
    };

    // vertically and horizontally center the complete diagram in the
    // available grid
    var optimizeGridPositions = function(nodes){

      //normalize top-bottom offset, so that top row is always y=0
      while (nodes.filter(n=>n.y < 0).length > 0){
        nodes.forEach(n=>n.y = n.y+1);
      }
      while (nodes.filter(n=>n.y <= 0).length === 0){
        nodes.forEach(n=>n.y = n.y-1);
      }

      // Get the current expansion of the diagram ...
      var maxExpansion = {
        left: 0,
        right: 0,
        top: 0,
        bottom: 0
      }
      nodes.forEach(n=>{
        if (n.x > maxExpansion.right) maxExpansion.right = n.x;
        if (n.x < maxExpansion.left) maxExpansion.left = n.x;

        if (n.y < maxExpansion.top) maxExpansion.top = n.y;
        if (n.y > maxExpansion.bottom) maxExpansion.bottom = n.y;
      });

      // ...and shift it horizontally for optimal display
      var width = maxExpansion.right - maxExpansion.left;
      var optimalDistanceToCenter = Math.floor(width/2);
      var shiftToRight = optimalDistanceToCenter - maxExpansion.right;

      // ...and shift it vertically
      var height = maxExpansion.bottom - maxExpansion.top;
      var optimalDistanceToCenter = Math.floor(height/2);
      var shiftDown = optimalDistanceToCenter - maxExpansion.bottom;

      nodes.forEach(n=>{
        n.x += shiftToRight
        n.y += shiftDown;
      });

    }


    var draw = function(node){

      // Set the dimensions of the diagram
      var div = document.querySelector("#branch-vis");
      // the width of the viewport is the maximum number of horizontal
      // nodes multiplied with the horizontal distance between nodes.
      var intrinsicWidth = MAXIMUM_HORIZONTAL_NODES * distanceX;
      //the height of the SVG viewbox is based on the number of vertical nodes
      //There is no vertical height limit.
      var intrinsicHeight = node.dimensions.currentHeight * distanceY + 20;

      // Assigns parent, children, grid position
      var nodesAndStubs = collectNodesAndStubs(node, null, [], []);
      var nodes = nodesAndStubs.nodes;
      var stubs = nodesAndStubs.stubs;

      optimizeGridPositions(nodes);

      // if we have an even number of rows, no row should be exactly in
      // the vertical middle. we shall add a vertical offset then.
      // this offset is half the vertical distance between nodes.
      var offsetY = (node.dimensions.currentHeight % 2 === 0) ? (distanceY / 2) : 0;

      //if we have an even number of columns, and if there are less nodes than
      // the maximum number of horizontal nodes, we shall add an offset, so that
      //the visualization is centered again, and there is NO single node in the
      //center.
      let offsetX = 0;
      if (
        (node.dimensions.currentWidth % 2 === 0)
        && (node.dimensions.currentWidth < MAXIMUM_HORIZONTAL_NODES)
      ){
        offsetX = distanceX / 2;
      }

      assignWidthAndHeight(
        nodes,
        intrinsicWidth / 2 + offsetX,
        intrinsicHeight / 2 + offsetY
      );
      //every node, except the root node, has a link
      var links = nodes.slice(1);

      // ****************** D3 section ***************************

      // append the svg object to the body of the page
      // appends a 'group' element to 'svg'
      var svg = d3
      .select("#branch-vis")
      .html("")
      .append("svg")
      .attr("viewBox", [0, 0, intrinsicWidth, intrinsicHeight].join(" "))
      .append("g");

      // Update the nodes...
      var i = 0;
      var node = svg
      .selectAll('g.node')
      .data(nodes, function(d) {
        return d.id || (d.id = ++i);
      })

      // Enter any new modes at the parent's previous position.
      var nodeEnter = node
      .enter()
      .append('g')
      .attr('class', 'node');


      // Add Circle for the "normal" nodes, which are not substitutions
      // and have a reference node (which means, they are not the central
      // Home Node)
      nodeEnter
      .filter(function(d){ return (!d.isSubstitution) && (d.referenceNode); })
      .append("a")
      .attr("xlink:href", function(d){
        return ZETTEL_URL_PREFIX + d.ekin;
      })
      //it seems, we have to set cursor for <a> in <svg> manually,
      //otherwise the cursor becomes only in Firefox a pointer, but not
      //in Chrome.
      .attr('cursor', 'pointer')
      .append('circle')
      .attr('class', 'node')
      .style("fill", "#fff")
      .style("stroke", "#77a")
      .style("stroke-width", "5")
      .attr('r', "27")
      .attr("transform", function(d) {
        return "translate(" + d.x + "," + d.y + ")";
      });

      //add mouseover text to node
      nodeEnter
      .append("title")
      .text(d => d.shortTitle);

      // ... and now for the central node
      nodeEnter
      .filter(function(d){ return (!d.referenceNode); })
      .append("a")
        .attr("xlink:href", function(d){
        return ZETTEL_URL_PREFIX + d.ekin;
      })
      .attr('cursor', 'pointer')
      .append('circle')
      .attr('class', 'node')
      .style("fill", "#fca")
      .style("stroke", "#000")
      .style("stroke-width", "8")
      .attr('r', "33")
      .attr("transform", function(d) {
        return "translate(" + d.x + "," + d.y + ")";
      })
      //add mouseover text to node
      .append("title")
      .text(d => d.shortTitle);


      // Add invisible circle without link for substitution nodes
      nodeEnter
      .filter(function(d){ return d.isSubstitution; })
      .append('circle')
      .attr('class', 'node')
      .style("fill", "#fff")
      .style("stroke-width", "0")
      .attr('r', "33")
      .attr("transform", function(d) {
        return "translate(" + d.x + "," + d.y + ")";
      });


      // Add labels for the (real) nodes
      nodeEnter
      .filter(d => !d.isSubstitution)
      .append("a")
      .attr("xlink:href", function(d){
        return ZETTEL_URL_PREFIX + d.ekin;
      })
      .attr('cursor', 'pointer')
      .append('text')
      .attr("dy", ".35em")
      .attr("text-anchor", "middle")
      .text(d => {
        if (d.shortTitle.length > 7){
          return "..." + d.shortTitle.substr(-5);
        } else {
          return d.shortTitle;
        }
      })
      .attr('font-size', "12")
      .style('font-family', "sans-serif")
      .attr('fill', "black")
      .attr("transform", function(d) {
        return "translate(" + d.x + "," + d.y + ")";
      })
      //add mouseover title to text label
      .append("title")
      .text(d => d.shortTitle);


      // Add labels for the substitutions
      nodeEnter
      .filter(d => d.isSubstitution)
      .append('text')
      .attr("dy", ".35em")
      .attr("text-anchor", "middle")
      .text(d => { return "..."})
      .attr('font-size', "26")
      .style('font-family', "sans-serif")
      .attr('fill', "#000")
      .attr("transform", function(d) {
        return "translate(" + d.x + "," + (d.y+5) + ")";
      });

      nodeEnter
      .filter(d => d.isSubstitution)
      .append('text')
      .attr("dy", ".35em")
      .attr("text-anchor", "middle")
      .text(d => { return d.length})
      .attr('font-size', "18")
      .style('font-family', "sans-serif")
      .attr('fill', "#000")
      .attr("transform", function(d) {
        return "translate(" + d.x + "," + (d.y-5) + ")";
      });


      var link = svg
      .selectAll('path.link')
      .data(links, function(d) { return d.id; })
      .enter()
      .insert('path', "g")
      .attr("class", "link")
      .attr("stroke", function(d) {
        if (d.relationToReferenceNode === "sameBranch") return "#999";
        if (d.relationToReferenceNode.includes("eingeschoben")) return "#faf";
        if (d.relationToReferenceNode.includes("ergaenzend")) return "#f88";
        if (d.relationToReferenceNode.includes("parallel")) return "#5fa";
      })
      .attr("stroke-width", "5")
      .attr('d', function(d){
        return `M ${d.x} ${d.y}, ${d.referenceNode.x} ${d.referenceNode.y}`
      });


      var stub = svg
      .selectAll('g.stub')
      .data(stubs, function(d) {
        return d.id || (d.id = ++i);
      })
      .enter()
      .filter(d => !d.subNodes)
      .insert('path', "g")
      .attr("class", "link")
      .attr("stroke", "#77a")
      .attr("stroke-width", "7")
      .attr('d', function(d){
        return `M ${d.origin.x} ${d.origin.y}, ${d.origin.x + 30} ${d.origin.y + 30}`
      });

    }

    draw(zettel);

  }


  render(){

    const self = this;

    return (
      <div className="zettel-branch-vis-div" id="branch-vis">
        {
          (!this.props.zettel.flags.hasBranchVisualization)
          ? <p style={{"textAlign": "center"}}>
            Für diesen Zettel ist noch keine Visualisierung der Zettelverknüpfung verfügbar.
          </p>
          : ""
        }
      </div>
    );
  }
}
